#!/bin/bash
# First, find out where we are
case $0 in
         /*)  SHELLFILE=$0 ;;
		./*)  SHELLFILE=${PWD}${0#.} ;;
        ../*) SHELLFILE=${PWD%/*}${0#..} ;;
          *)  SHELLFILE=$(type -P "$0") ; if [ "${SHELLFILE:0:1}" != "/" ]; then SHELLFILE="${PWD}/$SHELLFILE" ; fi ;;
esac

# readlink -m makes sure SHELLDIR is a simple absolute path
SHELLDIR=`readlink -m "${SHELLFILE%/*}"`

# 6.0: So the RT-STPS root directory should be two directories above. Make it an absolute path
RT_STPS_HOME=`readlink -m "$SHELLDIR/../../"`

ARCH=`uname -m`
if [ ${ARCH} == 'x86_64' ]; then

# 64-bit linux
HDF="$RT_STPS_HOME/lib/hdf-java-2.7/linux64/hdf-java/lib"
HDF_LIB="linux"

elif [ $(ARCH) == 'ia64' ]; then

# 64-bit itanium
HDF="$RT_STPS_HOME/lib/hdf-java-2.7/linux64i/hdf-java/lib"
HDF_LIB="linux"

else 

# 32-bit linux
HDF="$RT_STPS_HOME/lib/hdf-java-2.7/linux32/hdf-java/lib"
HDF_LIB="linux"

fi

######################################################################################
# Functions:
######################################################################################
# Complain and exit function

function printUsageAndExit () {
	echo "Usage:"
	echo "  packet_file_reader.sh"
	echo "REQUIRED parameters:"
	echo "  packetfile <path to packet file> \\ "
	echo "OPTIONAL parameters:"
	echo "  printall <true|false (default true): If true, prints all packets regardless of presence/absence of secondary header > \\ "
	echo "  CUCtime <true|false (default false): Select true if packet file contains unsegmented secondary header timestamps. Select false otherwise > \\ "
	echo "  timeoffset <(default 6): byte offset to packets' secondary header timestamp > \\ "
	echo ""
	echo "Example usage:"
	echo "  ./packet_file_reader.sh packetfile /home/user/data/P1570000.PDS"
	echo "  ./packet_file_reader.sh packetfile /home/user/data/P1540957.PDS printall true CUCtime true timeoffset 6"
	echo ""
	exit 1
}

######################################################################################
# Script starts here:
######################################################################################
# Initialize the input parameters
packetfile=""
printall="true"
CUCtime="false"
timeoffset="6"

# Parse the input arguments
while [[ "$1" != "" ]]; do
	key="$1"
	shift
	value="$1"
	shift
	case "$key" in
		"packetfile") packetfile="$value";;
		"printall") printall="$value";;
		"CUCtime") CUCtime="$value";;
		"timeoffset") timeoffset="$value";;
	esac
done

# Check if all require arguments were provided, and complain if any are missing
if [ -z "$packetfile" ]; then
	echo ""
	echo "ERROR - Path to packet file not provided"
	printUsageAndExit
fi

# 6.0: root no longer points to config so we don't have to maintain two separate rt-stps.dtd files. Now it's top-level RT-STPS directory.
java -Djava.library.path="$HDF/$HDF_LIB" -Droot="$RT_STPS_HOME" -cp "$RT_STPS_HOME/lib/rt-stps.jar:$RT_STPS_HOME/lib/nsls.jar:$HDF/fits.jar:$HDF/jhdf.jar:$HDF/jhdf4obj.jar:$HDF/jhdf5.jar:$HDF/jhdf5obj.jar:$HDF/jhdfobj.jar:$HDF/jhdfview.jar:$HDF/junit.jar:$HDF/netcdf.jar:$HDF/ext/fitsobj.jar:$HDF/ext/nc2obj.jar" gov.nasa.gsfc.drl.rtstps.core.output.rdr.tools.PacketReaderDriver "$packetfile" "$printall" "$CUCtime" "$timeoffset"
