#!/bin/bash
# First, find out where we are
case $0 in
         /*)  SHELLFILE=$0 ;;
		./*)  SHELLFILE=${PWD}${0#.} ;;
        ../*) SHELLFILE=${PWD%/*}${0#..} ;;
          *)  SHELLFILE=$(type -P "$0") ; if [ "${SHELLFILE:0:1}" != "/" ]; then SHELLFILE="${PWD}/$SHELLFILE" ; fi ;;
esac

# readlink -m makes sure SHELLDIR is a simple absolute path
SHELLDIR=`readlink -m "${SHELLFILE%/*}"`

# So the RT-STPS root directory should be one directory above
RT_STPS_HOME="$SHELLDIR/.."

ARCH=`uname -m`
if [ ${ARCH} == 'x86_64' ]; then

# 64-bit linux
HDF="$RT_STPS_HOME/lib/hdf-java-2.7/linux64/hdf-java/lib"
HDF_LIB="linux"

elif [ $(ARCH) == 'ia64' ]; then

# 64-bit itanium
HDF="$RT_STPS_HOME/lib/hdf-java-2.7/linux64i/hdf-java/lib"
HDF_LIB="linux"

else 

# 32-bit linux
HDF="$RT_STPS_HOME/lib/hdf-java-2.7/linux32/hdf-java/lib"
HDF_LIB="linux"

fi

######################################################################################
# Functions:
######################################################################################
# Complain and exit function

function printUsageAndExit () {
	echo "Usage:"
	echo "   stop.sh <server name>"
	echo "OPTIONAL arguments:"
	echo "   <server name> - the assigned name of an existing RT-STPS server instance, in double-quotes (default: \"A\") "
	echo ""
	echo "Example usage:"
	echo "  ./stop.sh"
	echo "  ./stop.sh \"rtstps2\""
	echo ""
	exit 1
}

######################################################################################
# Script starts here:
######################################################################################
# Initialize the input parameters to default values
servername="A"
rmiport="1099"
hostName="localhost"

# Now assign command line arguments to parameters, if applicable
# (Don't overwrite defaults if arguments weren't provided)
if [ ! -z "$1" ]; then
	servername="$1"
fi

if [ ! -z "$2" ]; then
	rmiport="$2"
fi

if [ ! -z "$3" ]; then
	hostName="$3"
fi

# This will contain the path to server_info file to delete (for non-JSW servers)
infoToDelete=""

# Attempt to parse server info if non-default servername is provided by the user
if [ ! -z "$1" ] && [ "$1" != "A" ] && [ "$hostName" == "localhost" ]; then
	# Check if this server exists as a JSW server. If so, source its server_info file
	if [ -d "$RT_STPS_HOME/jsw_$servername/" ]; then
		source "$RT_STPS_HOME/jsw_$servername/server_info"
	# No JSW instance was found for the given server name. Try to find a non-JSW instance
	elif [ -f "$RT_STPS_HOME/info/server_info_$servername" ]; then
		source "$RT_STPS_HOME/info/server_info_$servername"
		infoToDelete="$RT_STPS_HOME/info/server_info_$servername"
	# No instance was found for the given server name. Complain and exit
	else
		echo "ERROR - RT-STPS server named \"$servername\" does not exist!"
		exit 1
	fi
fi

java -Djava.library.path="$HDF/$HDF_LIB" -cp "$RT_STPS_HOME/lib/rt-stps.jar:$RT_STPS_HOME/lib/nsls.jar:$HDF/fits.jar:$HDF/jhdf.jar:$HDF/jhdf4obj.jar:$HDF/jhdf5.jar:$HDF/jhdf5obj.jar:$HDF/jhdfobj.jar:$HDF/jhdfview.jar:$HDF/junit.jar:$HDF/netcdf.jar:$HDF/ext/fitsobj.jar:$HDF/ext/nc2obj.jar" gov.nasa.gsfc.drl.rtstps.clients.Kill $hostName $servername $rmiport

# If the server killed was non-JSW, delete its corresponding server info file
if [ "$?" -eq 0 ]; then
	if [ ! -z "$infoToDelete" ]; then
		rm -f "$infoToDelete"
	fi
	echo "Stopped RT-STPS server instance \"$servername\""
fi
